<?php

namespace Drupal\commerce_stripe_payment_request_button\Controller;

use Drupal\commerce_order\Entity\OrderInterface;
use Drupal\commerce_price\Price;
use Drupal\commerce_shipping\Entity\Shipment;
use Drupal\commerce_shipping\ShipmentItem;
use Drupal\Component\Serialization\Json;
use Drupal\physical\Weight;
use Drupal\physical\WeightUnit;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;

class PaymentRequestButtonShipping {

  public function handleShipping(Request $request) {

    // Transform JSON values to array
    $values = Json::decode($request->getContent());
    $shipping_info = $values["shippingAddress"];
    $order_id = $values["order_id"];

    $carts = \Drupal::getContainer()
      ->get('commerce_cart.cart_provider')
      ->getCarts();
    $active_cart = array_filter($carts, function (OrderInterface $cart) use ($order_id) {
      return $cart->id() == $order_id;
    });
    $order = reset($active_cart);


    if ($order->hasItems()) {

      // Check if we have some made before shipments
      if ($order->get('shipments')->count() != 0) {
        // delete them or something
        $shipments = $order->shipments->referencedEntities();
        if (!$shipments) {
          /** @var \Drupal\commerce_shipping\PackerManagerInterface $packer_manager */
          $packer_manager = \Drupal::service('commerce_shipping.packer_manager');
          list($shipments) = $packer_manager->packToShipments($order, $this->buildCustomerProfile($order), $shipments);
        }
        /** @var \Drupal\commerce_shipping\Entity\ShipmentInterface $first_shipment */
        $first_shipment = $shipments[0];
        /** @var \Drupal\profile\Entity\ProfileInterface $profile */
        $profile = $first_shipment->getShippingProfile() ?: $this->buildCustomerProfile($order);

        // This is a hack but shipments with empty amounts is crashing other
        // contrib modules.
        // Ideally, we shouldn't have to pack the shipments ourselves...
        if (!$first_shipment->getAmount()) {
          $shipment_amount = Price::fromArray([
            'number' => 0,
            'currency_code' => $order->getTotalPrice()->getCurrencyCode(),
          ]);
          $first_shipment->setAmount($shipment_amount);
        }
        // TODO do something with above
      }
      else {

        $first_shipment = Shipment::create([
          'type' => 'default',
          'order_id' => $order_id,
          'title' => 'Shipment',
          'amount' => new Price('5', 'USD'),
          'state' => 'ready',
        ]);

        // Loop through order items and add them to shipment
        foreach ($order->getItems() as $order_item) {
          $quantity = $order_item->getQuantity();
          $purchased_entity = $order_item->getPurchasedEntity();

          if ($purchased_entity->get('weight')->isEmpty()) {
            $weight = new Weight(1, WeightUnit::GRAM);
          }
          else {
            $weight_item = $purchased_entity->get('weight')->first();
            $weight = $weight_item->toMeasurement();
          }

          $shipment_item = new ShipmentItem([
            'order_item_id' => $order_item->id(),
            'title' => $purchased_entity->label(),
            'quantity' => $quantity,
            'weight' => $weight->multiply($quantity),
            'declared_value' => $order_item->getTotalPrice(),
          ]);
          $first_shipment->addItem($shipment_item);
        }

        $shipping_method_storage = \Drupal::entityTypeManager()->getStorage('commerce_shipping_method');
        $shipping_methods = $shipping_method_storage->loadMultipleForShipment($first_shipment);

        // Find rate, we use the first one, pretending there is only one we want to use, fixed rate for order
        $shipping_method_plugin = reset($shipping_methods)->getPlugin();
        $shipping_rates = $shipping_method_plugin->calculateRates($first_shipment);

        $amount = reset($shipping_rates)->getAmount();

        $first_shipment->setAmount($amount);
        $first_shipment->setShippingMethod(reset($shipping_methods));
        $first_shipment->save();

        $order->set('shipments', [$first_shipment]);
        $order->save();
      }
      // We only get the full name from Apple/Google pay, so we need to "guess" the given
      // name and the family name.
      $names = explode(' ', $shipping_info['recipient']);
      $given_name = array_shift($names);
      $family_name = implode(' ', $names);

      $profile = $this->buildCustomerProfile($order);
      $profile->address->given_name = $given_name;
      $profile->address->family_name = $family_name;

      $profile->address->given_name = $given_name;
      $profile->address->family_name = $family_name;
      $profile->address->country_code = $shipping_info['country'];
      $profile->address->locality = $shipping_info['city'];
      $profile->address->administrative_area = $shipping_info['dependentLocality'];
      $profile->address->postal_code = $shipping_info['postalCode'];
      $profile->address->address_line1 = $shipping_info["addressLine"][0];
      $profile->address->address_line2 = isset($shipping_info["addressLine"][1])?:"";
      $profile->save();

      $first_shipment->setShippingProfile($profile);
      $first_shipment->save();
      $order->set('shipments', $first_shipment);
      $order->save();
    }


    $response['shippingOptions'] = [
      [
        'id' => 'free',
        'label' => 'free-shipping',
        'detail' => 'Arrives soon, yeah right',
        'amount' => 0,
      ],
    ];

    return new JsonResponse($response);
  }


  protected function buildCustomerProfile(OrderInterface $order) {
    return \Drupal::entityTypeManager()->getStorage('profile')->create([
      'uid' => $order->getCustomerId(),
      'type' => 'customer',
    ]);
  }

  protected function getShippingProfile() {
    $shipping_profile = NULL;
    /** @var \Drupal\commerce_shipping\Entity\ShipmentInterface $shipment */
    foreach ($this->order->shipments->referencedEntities() as $shipment) {
      $shipping_profile = $shipment->getShippingProfile();
      break;
    }
    if (!$shipping_profile) {
      $shipping_profile = $this->entityTypeManager->getStorage('profile')
        ->create([
          'type' => 'customer',
          'uid' => $this->order->getCustomerId(),
        ]);
    }

    return $shipping_profile;
  }
}
